/* eslint-env node */
const path = require('path');
const http = require('http');

const httpProxy = require('http-proxy');
const proxy = httpProxy.createProxyServer({});



async function init() {
  const host = 'localhost';
  let port = 9001;
  const max = 9101;
  while(port <= max) {
    const server = http.createServer(requestListener);
    const res = await new Promise((resolve) => {
      server.once('error', (err) => {
        console.log(`${err.message}, retring ${++port}.`);
        resolve(false);
      });
      server.listen(port, () => {
        resolve(true);
      });
    });
    if (res) break;
  }
  if (port <= max) {

    console.log(`A proxy server is listening at: \x1b[32mhttp://${host}:${port}\x1b[0m`, '\n');
    return `${host}:${port}`;
  }
  throw new Error('start a proxy server error: no enough ports to use.');
}

init();

/**
 * Server listener.
 * @param {object} req
 * @param {object} res
 */
async function requestListener(req, res) {
  // /https/jsonplaceholder.typicode.com/photos/1?bypass=1
  console.log('req:', req.url);
  req.url = '/photos/1?bypass=1';
  console.log('req:', req.url);
  return proxy.web(req, res, {
    changeOrigin: true,
    target: 'https://jsonplaceholder.typicode.com',
    headers: {  },
    cookieDomainRewrite: '',
    followRedirects: true,
  }, function(err) {
    console.log(`proxy error[${req.url}]: `, err.message);
    serverError(res, 'proxy error: ' + err.message);
  });
}



/**
 * Send an internal server error.
 * @param {object} res
 * @param {string} body
 */
function serverError(res, body = 'Internal Server Error.') {
  res.writeHead(500, {});
  res.end(body, 'utf8');
}

process.on('unhandledRejection', (reason, p) => {
  console.error(reason, 'Unhandled Rejection at Promise', p);
}).on('uncaughtException', err => {
  console.error(err, 'Uncaught Exception thrown');
  process.exit(1);
});


